#!/bin/sh

#===================various function implementation===========


action_for_ubuntu19()
{
	#use jpeg library in install package
	cp -f libs/std/libjpeg.so.62 libs/
	#use system's libncurses, package do not contain library, but version mismatch, this may have problem
	ln -s /usr/lib/x86_64-linux-gnu/libncurses.so.6 libs/libncurses.so.5
	#if not find libgtk-x11-2.0.so.0, install libgtk2.0-0_2.24.32-3ubuntu1_amd64.deb and it's dependency
	dpkg -S libgtk-x11-2.0.so.0 > /dev/null 2>&1
	if [ "$?" != "0" ]
	then
		echo -n "Installing libgtk2.0-0..."
		dpkg -i  ./libs/linux_release_packages/ubuntu/19/libgtk2.0-common_2.24.32-3ubuntu1_all.deb >/dev/null 2>&1
		dpkg -i  ./libs/linux_release_packages/ubuntu/19/libgtk2.0-0_2.24.32-3ubuntu1_amd64.deb >/dev/null 2>&1
		echo "... Complete!"
	fi
	#add to system application repository
	cp -f iNodeClient.desktop /usr/share/applications/		
	return 0
}

action_for_ubuntu18()
{
	#add to system application repository
	cp -f iNodeClient.desktop /usr/share/applications/		
	return 0
}

action_for_centos()
{
	#add to system application repository
	cp -f iNodeClient.desktop /usr/share/applications/
	if [ "$OS_CENTOS8" != "" -a -r /usr/lib64/libncurses.so.6 ]
	then
	ln -s /usr/lib64/libncurses.so.6 libs/libncurses.so.5	
	fi
	return 0
}

action_for_debian()
{
   if [ "$OS_DEBIAN" != "" -a  -r /lib/x86_64-linux-gnu/libncurses.so.6 ]
     then
	    ln -s /lib/x86_64-linux-gnu/libncurses.so.6 ./libs/libncurses.so.5 
   fi

    dpkg -S libgtk-x11-2.0.so.0 > /dev/null 2>&1
		if [ "$?" != "0" ]
	      then
		   echo -n "Installing libgtk2.0-0..."
		  sudo dpkg -i  ./libs/linux_release_packages/debian/libgtk2.0-common_2.24.32-3_all.deb >/dev/null 2>&1
		  sudo dpkg -i  ./libs/linux_release_packages/debian/libgtk2.0-0_2.24.32-3_amd64.deb >/dev/null 2>&1
		  echo "... Complete!"
	    fi
	cp iNodeClient.desktop /usr/share/applications/

}

action_for_KylinV10()
{
	# Kylin v10 dosen't has libjpeg.so.62
	cp -f libs/std/libjpeg.so.62 libs/
}

special_action_for_os()
{
	case "$1" in
		"Ubuntu 19"* )
			action_for_ubuntu19
		;;
		"Ubuntu 18"* )
			action_for_ubuntu18
		;;
		"CentOS"* )
			action_for_centos
		;;
		*"Debian"* )
			action_for_debian
		;;
		"Kylin V10"* )
			action_for_KylinV10
		;;
		*)
			echo "Nothing special for $1"
		;;
	esac		
	return 0
}

ubuntu_terminate_process()
{
	ubuntu_os=`grep -e "^.*Ubuntu.*$" /etc/issue`
	case "$ubuntu_os" in 
		"Ubuntu 19"* )
			sudo pkill -SIGKILL -f $1 >  /dev/null 2>&1
		;;
		*)
		    sudo killall -9 $1 > /dev/null 2>&1
		;;
	esac
}

linux_terminate_process()
{
	os=`grep -e "^.*Ubuntu.*$" -e "^.*Fedora.*$" -e "^.*Rocky.*$" -e "^.*Deepin.*$" -e "^.*Linx.*$" /etc/issue`
	case "$os" in
	Ubuntu*)
		ubuntu_terminate_process $1
	;;
	*)
		sudo killall -9 $1 > /dev/null 2>&1
	;;
	esac
	return 0
}


#===================main script code=========================
#check euid
if [ `id -u -r` != "0" ]
then
	echo "Please use \"sudo\" launch install script"
	exit 0
fi

CURRENT=`pwd`
#check install path
CHECKRESULT=`echo $CURRENT|grep '[^a-zA-Z0-9/(){}_. -]'`
if [ "$CHECKRESULT" != "" ];then
    echo "Invalid iNode client installation directory name. "
    echo "The directory name can contain only upper-case/lower-case English letters, digits, spaces, and the following characters:(){}/._-."
    exit 0
fi

INODE_CFG="/etc/iNode/inodesys.conf"

if [ -r "$INODE_CFG" ];then
    LINE=`cat $INODE_CFG`
    INSTALL_DIR=${LINE##*INSTALL_DIR=}
fi

if [ "$INSTALL_DIR" != "" ];then
    echo "iNode has been installed on the path $INSTALL_DIR."
    if [ "$INSTALL_DIR" != "$CURRENT" ]  
    then
        echo -n "This operation will remove iNode. Are you sure to continue?[input Y/y to continue]:"
	    read ISCONTINUE    
	    if [ "$ISCONTINUE" != "Y" -a "$ISCONTINUE" != "y" ]; then                   
	        exit 0        
	    fi
	    cd "$INSTALL_DIR"        
	    ./uninstall.sh
	    cd "$CURRENT"
	    INSTALL_DIR=""
    else  
        exit 0
    fi        
fi

IfExistMon=`ps awx -o command|awk -F/ '{print $NF}'|grep -x iNodeMon`
if [ "$IfExistMon" != "" ]
then
    if [ -n "$INSTALL_DIR" ]
    then
        "$INSTALL_DIR/iNodeMon" -k
    fi
        
    Sec=0
    while [ 1 ]
    do
        IfExistMon=`ps awx -o command|awk -F/ '{print $NF}'|grep -x iNodeMon`
	if [ "$IfExistMon" != "" ]
        then
	    sleep 1
	    Sec=`expr $Sec + 1`

	    if [ "$Sec" -gt 10 ]
	    then
	        #killall -9 iNodeMon
			linux_terminate_process iNodeMon
	    fi
	else
	    break
	fi
    done
fi

IfExistAuth=`ps awx -o command|awk -F/ '{print $NF}'|grep -x AuthenMngService`
if [ "$IfExistAuth" != "" ]
then
    if [ -n "$INSTALL_DIR" ]
    then
        "$INSTALL_DIR/AuthenMngService" -k
    fi
    
    Sec=0
    while [ 1 ]
    do
        IfExistAuth=`ps awx -o command|awk -F/ '{print $NF}'|grep -x AuthenMngService`
	if [ "$IfExistAuth" != "" ]
        then
	    sleep 1
	    Sec=`expr $Sec + 1`

	    if [ "$Sec" -gt 10 ]
	    then
	        #killall -9 AuthenMngService
			linux_terminate_process AuthenMngService
	    fi
	else
	    break
	fi
    done
fi

IfExistUI=`ps awx -o command|awk -F/ '{print $NF}'|grep -x iNodeClient`
if [ "$IfExistUI" != "" ]
then
    sleep 3
    #killall -9 iNodeClient
	linux_terminate_process iNodeClient
fi

if [ ! -r "/etc/iNode" ]
then
mkdir /etc/iNode
fi

if [ ! -r "./clientfiles" ]
then
mkdir ./clientfiles
fi

if [ ! -r "./conf" ]
then
mkdir ./conf
fi

if [ ! -r "./log" ]
then
mkdir ./log
fi

if [ ! -r "./libs/std" ]
then
tar -C ./libs -xzf ./libs/linux64_std.tar.gz
fi

if [ ! -r "./libs/linux_release_packages" ]
then
tar -C ./libs -xzf ./libs/linux_release_packages.tar.gz
fi

INODE_CFG="/etc/iNode/inodesys.conf"

if [ -r "$INODE_CFG" ];then
    LINE=`cat $INODE_CFG`
    INSTALL_DIR=${LINE##*INSTALL_DIR=}
fi

if [ -z "$INSTALL_DIR" ]; then
    echo INSTALL_DIR=$CURRENT >> /etc/iNode/inodesys.conf
fi



if [  -r "/usr/lib64" ]
then 
    if [ ! -r "/usr/lib64/libstdc++.so.6" ]
    then
        cp -fr ./libs/std/libstdc++.so.6 ./libs/libstdc++.so.6
    fi
    if [ ! -r "/usr/lib64/libstdc++.so.6.0.13" ]
    then
        cp -fr ./libs/std/libstdc++.so.6.0.13 ./libs/libstdc++.so.6.0.13
    fi
	  
    if [ ! -r "/usr/lib64/libtiff.so.3" ]
    then
        cp -fr ./libs/std/libtiff.so.3 ./libs/libtiff.so.3
    fi
    if [ ! -r "/usr/lib64/libtiff.so.3.9.4" ]
    then
        cp -fr ./libs/std/libtiff.so.3.9.4 ./libs/libtiff.so.3.9.4
    fi
    if [ ! -r "/usr/lib64/libpangox-1.0.so.0" ]
    then
        cp -fr ./libs/std/libpangox-1.0.so.0 ./libs/libpangox-1.0.so.0
    fi
    
    if [ ! -r "/usr/lib64/libpng12.so" ]
    then
        cp -fr ./libs/std/libpng12.so ./libs/libpng12.so
    fi
    if [ ! -r "/usr/lib64/libpng12.so.0" ]
    then
        cp -fr ./libs/std/libpng12.so.0 ./libs/libpng12.so.0
    fi
    if [ ! -r "/usr/lib64/libpng12.so.0.44.0" ]
    then
        cp -fr ./libs/std/libpng12.so.0.44.0 ./libs/libpng12.so.0.44.0
    fi


        if [ ! -r "/usr/lib/libstdc++.so.6" ]
	then
	cp -fr ./libs/std/libstdc++.so.6 ./libs/
	fi
	if [ ! -r "/usr/lib/libstdc++.so.6.0.13" ]
	then
	cp -fr ./libs/std/libstdc++.so.6.0.13 ./libs/libstdc++.so.6.0.13
	fi
	  
	if [ ! -r "/usr/lib/libtiff.so.3" ]
	then
	cp -fr ./libs/std/libtiff.so.3 ./libs/libtiff.so.3
	fi
	if [ ! -r "/usr/lib/libtiff.so.3.9.4" ]
	then
	cp -fr ./libs/std/libtiff.so.3.9.4 ./libs/libtiff.so.3.9.4
	fi

	
elif [ -r "/usr/lib" ]
then

	if [ ! -r "/usr/lib/libstdc++.so.6" ]
	then
	cp -fr ./libs/std/libstdc++.so.6 ./libs/
	fi
	if [ ! -r "/usr/lib/libstdc++.so.6.0.13" ]
	then
	cp -fr ./libs/std/libstdc++.so.6.0.13 ./libs/libstdc++.so.6.0.13
	fi
	  
	if [ ! -r "/usr/lib/libtiff.so.3" ]
	then
	cp -fr ./libs/std/libtiff.so.3 ./libs/libtiff.so.3
	fi
	if [ ! -r "/usr/lib/libtiff.so.3.9.4" ]
	then
	cp -fr ./libs/std/libtiff.so.3.9.4 ./libs/libtiff.so.3.9.4
	fi
	
	if [ ! -r "/usr/lib/libpng12.so" ]
    then
        cp -rf ./libs/std/libpng12.* ./libs/
    fi
	
	if [ ! -r "/usr/lib/libjpeg.so" ]
    then
        cp -rf ./libs/std/libjpeg.* ./libs/
    fi
	

else
	cp -rf ./libs/std/libpng12* ./libs/
	cp -rf ./libs/std/libjpeg* ./libs/
fi

if [ -r "./custom/DeskIconLinux.png" ]
then
rm -f ./resource/iNodeClient.png
cp -f ./custom/DeskIconLinux.png ./resource/iNodeClient.png
fi

sed -i "s:@INSTALL_PATH:$CURRENT:g" ./iNodeClient.desktop
sed -i "s:@INSTALL_PATH:$CURRENT:g" ./iNodeClient.sh

chmod 755 ./AuthenMngService
chmod 755 ./iNodeMon
chmod 755 ./renew.ps
chmod 755 ./enablecards.ps
chmod 755 ./iNodeClient.desktop
chmod 755 ./iNodeClient.sh
chmod 755 ./iNodeClient.sh
chmod 755 ./addRun.sh
chmod 755 ./delRun.sh
chmod 777 ./clientfiles
chmod 777 ./clientfiles/8021
chmod 777 ./clientfiles/5020
chmod 777 ./clientfiles/7000
chmod -R 777 ./conf

#=================OS Type====================
OS_UBUNTU=`cat /etc/issue | grep 'Ubuntu'`
OS_ROCKY=`cat /etc/issue | grep 'Rocky'`
OS_DEEPIN=`cat /etc/issue | grep 'Deepin'`
OS_UBUNTU_18=`cat /etc/issue | grep 'Ubuntu 18'`
OS_UBUNTU_19=`cat /etc/issue | grep 'Ubuntu 19'`
OS_UBUNTU_20=`cat /etc/issue | grep 'Ubuntu 20'`
OS_DEBIAN=`uname -a | grep 'Debian'`
OS_LINX=`cat /etc/issue | grep 'Linx'`

OS_Kylin=`cat /etc/issue | grep 'Kylin'`
OS_UOS=`cat /etc/issue | grep 'uos'`
OS_UOS_NEW=`cat /etc/issue | grep -i 'UnionTech OS'`

if [ -r "/etc/redhat-release" ]
then
OS_FEDORA=`cat /etc/redhat-release | grep 'Fedora'`
OS_CENTOS=`cat /etc/redhat-release | grep 'CentOS'`
OS_CENTOS8=`cat /etc/redhat-release | grep 'CentOS Linux release 8.'`
OS_REDHAT=`cat /etc/redhat-release | grep 'Red Hat'`
fi

if [ "$OS_ROCKY" != "" -o "$OS_LINX" != "" -o "$OS_Kylin" != "" -o "$OS_UOS_NEW" != "" -o "$OS_UOS" != "" ]
then
    echo "1" > conf/Tray.txt
else
    echo "0" > conf/Tray.txt
fi


if [ -r "/etc/os-release" ]
then
OS_SUSE=`cat /etc/os-release | grep 'SUSE'`
fi

if [ "$OS_FEDORA" != "" -a -r /usr/lib64/libncurses.so.6 ]
then
    ln -s /usr/lib64/libncurses.so.6 libs/libncurses.so.5	
fi

if [ "$OS_REDHAT" != "" -a -r /usr/lib64/libncurses.so.6 ]
then
    ln -s /usr/lib64/libncurses.so.6 libs/libncurses.so.5	
fi

if [ "$OS_UBUNTU_20" != "" ]
then
    cp -f libs/std/libjpeg.so.62 libs/
	#use system's libncurses, package do not contain library, but version mismatch, this may have problem
	ln -s /usr/lib/x86_64-linux-gnu/libncurses.so.6 libs/libncurses.so.5
	cp -f iNodeClient.desktop /usr/share/applications/
fi
	
if [ "$OS_LINX" != "" ]
then

    if [ ! -d "/var/lock/subsys" ]
    then
        mkdir -p /var/lock/subsys
    fi
fi
#launch special action for that os 
OS_ISSUE=`grep -e "^.*Ubuntu.*$" -e "^.*Fedora.*$" -e "^.*Rocky.*$" -e "^.*Deepin.*$" -e "^.*Linx.*$" -e "^.*Kylin.*$" /etc/issue`

if [ "$OS_ISSUE" = "" ]
then
    OS_ISSUE="$OS_CENTOS"
fi

if [ "$OS_ISSUE" = "" ]
then
    OS_ISSUE="$OS_DEBIAN"
fi


if [ "$OS_FEDORA" != "" ]
then
    export PATH=$PATH:/sbin
fi

if [ ! -r "/usr/lib/libtiff.so.3" ]
then
    cp -fr ./libs/std/libtiff.so.3 ./libs/libtiff.so.3
fi

if [ "$OS_DEEPIN" != "" -a  -r /lib/x86_64-linux-gnu/libncurses.so.6 ]
then
	ln -s /lib/x86_64-linux-gnu/libncurses.so.6 ./libs/libncurses.so.5 
fi

if [ "$OS_UBUNTU" != "" -o "$OS_DEEPIN" != "" ]
then
    mv -f ./iNodeAuthService_ubuntu /etc/init.d/iNodeAuthService
    rm -f ./iNodeAuthService
else
    mv -f ./iNodeAuthService /etc/init.d
    rm -f ./iNodeAuthService_ubuntu
fi

chmod 755 /etc/init.d/iNodeAuthService

if [ -r /etc/rc.local ]
then 
    cp -fr /etc/rc.local /etc/rc.local.bak
    sed -e '/^exit 0$/d' /etc/rc.local > /etc/rc.temp
    echo "/etc/init.d/iNodeAuthService start" >> /etc/rc.temp
    echo "exit 0" >> /etc/rc.temp
    mv -f /etc/rc.temp /etc/rc.local
    chmod 755 /etc/rc.local
else
    echo "#!/bin/bash" > /etc/rc.local
	echo "/etc/init.d/iNodeAuthService start" >> /etc/rc.local
	echo "exit 0" >> /etc/rc.local
	chmod 755 /etc/rc.local
fi 


if [ -d /etc/rc.d ]
then 
     ln -s /etc/init.d/iNodeAuthService /etc/rc.d/rc3.d/S080iNodeAuthService > /dev/null 2>&1
     ln -s /etc/init.d/iNodeAuthService /etc/rc.d/rc5.d/S080iNodeAuthService > /dev/null 2>&1
     ln -s /etc/init.d/iNodeAuthService /etc/rc.d/rc0.d/K01iNodeAuthService > /dev/null 2>&1
     ln -s /etc/init.d/iNodeAuthService /etc/rc.d/rc1.d/K01iNodeAuthService > /dev/null 2>&1
	 ln -s /etc/init.d/iNodeAuthService /etc/rc.d/rc2.d/K01iNodeAuthService > /dev/null 2>&1
     ln -s /etc/init.d/iNodeAuthService /etc/rc.d/rc4.d/K01iNodeAuthService > /dev/null 2>&1
     ln -s /etc/init.d/iNodeAuthService /etc/rc.d/rc6.d/K01iNodeAuthService > /dev/null 2>&1  

     chkconfig --add iNodeAuthService
     chkconfig --level 2345 iNodeAuthService on
     chkconfig --level 016 iNodeAuthService off
     chkconfig iNodeAuthService on	 
else
     ln -s /etc/init.d/iNodeAuthService /etc/rc3.d/S080iNodeAuthService > /dev/null 2>&1
     ln -s /etc/init.d/iNodeAuthService /etc/rc5.d/S080iNodeAuthService > /dev/null 2>&1
     ln -s /etc/init.d/iNodeAuthService /etc/rc0.d/K01iNodeAuthService > /dev/null 2>&1
     ln -s /etc/init.d/iNodeAuthService /etc/rc1.d/K01iNodeAuthService > /dev/null 2>&1
	 ln -s /etc/init.d/iNodeAuthService /etc/rc2.d/K01iNodeAuthService > /dev/null 2>&1
     ln -s /etc/init.d/iNodeAuthService /etc/rc4.d/K01iNodeAuthService > /dev/null 2>&1
     ln -s /etc/init.d/iNodeAuthService /etc/rc6.d/K01iNodeAuthService > /dev/null 2>&1
fi


if [ "$OS_ROCKY" != "" ]
then    
    cp -frH ./libs/std/libjpeg.so.62 ./libs/ > /dev/null 2>&1
    cp -fr ./libs/rocky/gdk-pixbuf.loaders /etc/gtk-2.0/    
fi

SELINUX_FLAG=`getenforce 2>/dev/null | grep -x -i enforcing`
if [ "$SELINUX_FLAG" != "" ]
then

chcon -t textrel_shlib_t ./libs/libCoreUtils.so
chcon -t textrel_shlib_t ./libs/libImplAv.so
chcon -t textrel_shlib_t ./libs/libOesisCore.so
fi

if [ "$OS_UBUNTU" != "" ];then
    if [ -d "/var/lib/locales/supported.d/" ];then
	    if [  ! -f "/var/lib/locales/supported.d/zh-inode" ];then
	        echo "zh_CN.GB2312 GB2312" > /var/lib/locales/supported.d/zh-inode
	        locale-gen > /dev/null 2>&1
	    fi
	fi
fi


if [ "$OS_ROCKY" != "" ]
then
    pango-querymodules > '/etc/pango/pango.modules'
fi

special_action_for_os "$OS_ISSUE"

if [ "$OS_UBUNTU_18" != "" -o "$OS_UBUNTU_19" != "" ]
then
./ubuntu18.sh
fi

if [ "$OS_SUSE" != "" ]
then 
   ln -s /lib64/libncurses.so.6 ./libs/libncurses.so.5
   cp -f libs/std/libjpeg.so.62 libs/
fi

/etc/init.d/iNodeAuthService start
rm -f ./install_64.sh

